screen = {
	width = love.graphics.getWidth(),
	height = love.graphics.getHeight()
}

function love.load()
	love.filesystem.setIdentity("Conway's game of life")
	--Loading
	slider = require "slider"
	button = require "button"

	--Various Variables
	alt = false
	sim = false
	mx, my = 0, 0
	t = 0

	local w, h, f = love.window.getMode()
	local m = 60
	if f.refreshrate > 0 then
		m = f.refreshrate
	end

	--Simulation Settings
	updateRate = m
	scale = 1
	deadColor = {54, 211, 120, 255}
	aliveColor = {32, 32, 32, 255}

	--Setup
	createGUI()
	love.graphics.setDefaultFilter("nearest", "nearest")
	canvas = {
		love.graphics.newCanvas(screen.width / scale, (screen.height - boxHeight) / scale ), --Shader canvas
		love.graphics.newCanvas(screen.width / scale, (screen.height - boxHeight) / scale ), --Shader canvas
		love.graphics.newCanvas(screen.width, screen.height)                                 --GUI canvas
	}
	love.graphics.setLineStyle("rough")
	love.graphics.setLineWidth(1)
	shader:send("aliveColor", aliveColor)
	shader:send("deadColor", deadColor)
	clearCanvas(1)
	clearCanvas(2)
	shader:send("size", {screen.width / scale, screen.height / scale})
	wave:send("size", {screen.width / scale, screen.height / scale})
	nt = love.timer.getTime()
	cells = canvas[1]:getWidth() * canvas[1]:getHeight()
	generation = 0
end

function noise()
	love.graphics.setCanvas(canvas[1])
	for y=1, canvas[1]:getHeight() do
		for x=1, canvas[1]:getWidth() do
			local r = math.random()
			if r > 0.5 then
				love.graphics.setColor(aliveColor)
				love.graphics.points(x, y)
			end
		end
	end
	love.graphics.setCanvas()
end

function createGUI()
	local w, h, f = love.window.getMode()
	boxHeight = math.floor(screen.height * 0.1)

	--Sliders
	local m = 60
	if f.refreshrate > 0 then
		m = f.refreshrate
	end
	updateSlider = slider.new(10, boxHeight * 0.1, math.floor(screen.width * 0.3), math.floor(boxHeight * 0.2), updateRate, 10, m, true)
	scaleSlider = slider.new(10, boxHeight * 0.55, math.floor(screen.width * 0.3), math.floor(boxHeight * 0.2), scale, 1, 10, true)

	--Buttons
	applyButton = button.new("apply\n(enter)", math.floor(screen.width * 0.32), boxHeight * 0.1, math.floor(screen.width * 0.08), math.floor(boxHeight * 0.8), applySettings, true)
	clearButton = button.new("clear\n(c)", math.floor(screen.width * 0.91), boxHeight * 0.1, math.floor(screen.width * 0.08), math.floor(boxHeight * 0.8), clear, true)
	toggleButton = button.new("start\n(space)", math.floor(screen.width * 0.71), boxHeight * 0.1, math.floor(screen.width * 0.08), math.floor(boxHeight * 0.8), toggleSimulation, true)
	stepButton = button.new("step\n(s)", math.floor(screen.width * 0.81), boxHeight * 0.1, math.floor(screen.width * 0.08), math.floor(boxHeight * 0.8), step, true)
end

function step()
	if not sim then
		sim = true
		love.draw()
		sim = false
	end
end

function applySettings(resize)
	updateRate = updateSlider:getValue()
	if scale ~= scaleSlider:getValue() or resize then
		scale = scaleSlider:getValue()
		canvas[1] = love.graphics.newCanvas(screen.width / scale, (screen.height - boxHeight) / scale )
		canvas[2] = love.graphics.newCanvas(screen.width / scale, (screen.height - boxHeight) / scale )
		cells = canvas[1]:getWidth() * canvas[1]:getHeight()
		clear()
		shader:send("size", {screen.width / scale, screen.height / scale})
	end
end

function clear()
	clearCanvas(1)
	clearCanvas(2)
	generation = 0
end

function toggleSimulation()
	sim = not sim
	if sim then
		toggleButton.text = "stop\n(space)"
	else
		toggleButton.text = "start\n(space)"
	end
end

function drawGUI()
	love.graphics.setCanvas(canvas[3])
	love.graphics.clear()
	love.graphics.setColor(aliveColor)
	love.graphics.rectangle("fill", 0, 0, screen.width, boxHeight)

	love.graphics.setColor(deadColor)

	updateSlider:draw()
	local sx, sy = updateSlider:getPosition()
	love.graphics.print("Frame Limiter: "..updateSlider:getValue(), sx, sy + updateSlider:getHeight())

	scaleSlider:draw()
	local sx, sy = scaleSlider:getPosition()
	love.graphics.print("Scale: "..scaleSlider:getValue(), sx, sy + scaleSlider:getHeight())

	applyButton:draw()
	clearButton:draw()
	toggleButton:draw()
	stepButton:draw()

	love.graphics.setColor(aliveColor)
	love.graphics.printf("fps: "..love.timer.getFPS().."  |  Total Cells: "..cells.."	|	Generation: "..generation, 0, screen.height - 16, screen.width, "center")

	love.graphics.setCanvas()
end

function clearCanvas(i)
	love.graphics.setCanvas(canvas[i])
	love.graphics.clear()
	love.graphics.setColor(deadColor)
	love.graphics.rectangle("fill", 0, 0, screen.width / scale, (screen.height - boxHeight) / scale )
	love.graphics.setCanvas()
end

function love.update(dt)
	nt = nt + (1 / updateRate)
	t = t + (1 / updateRate)
	wave:send("t", t)
	updateSlider:update(dt)
	scaleSlider:update(dt)
	applyButton:update(dt)
	clearButton:update(dt)
	toggleButton:update(dt)
	stepButton:update(dt)
end

function love.draw()
	local c1, c2 = 1, 2
	if alt then
		c1 = 2
		c2 = 1
	end

	love.graphics.setCanvas(canvas[c1])
	if love.mouse.isDown(1) then
		if love.mouse.getY() > boxHeight then
			love.graphics.setColor(aliveColor)
			if mx == love.mouse.getX() and my == love.mouse.getY() then
				love.graphics.points(love.mouse.getX() / scale, (love.mouse.getY() - boxHeight) / scale )
			else
				love.graphics.line(love.mouse.getX() / scale, (love.mouse.getY() - boxHeight) / scale, mx / scale, (my - boxHeight) / scale)
			end
		end
	elseif love.mouse.isDown(2) then
		if love.mouse.getY() > boxHeight then
			love.graphics.setColor(deadColor)
			if mx == love.mouse.getX() and my == love.mouse.getY() then
				love.graphics.points(love.mouse.getX() / scale, (love.mouse.getY() - boxHeight) / scale )
			else
				love.graphics.line(love.mouse.getX() / scale, (love.mouse.getY() - boxHeight) / scale, mx / scale, (my - boxHeight) / scale)
			end
		end
	end	

	shader:send("map", canvas[c1])

	love.graphics.setCanvas(canvas[c2])
	
	if sim then 
		love.graphics.setShader(shader)
		generation = generation + 1 
	end
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(canvas[c1], 0, 0)
	love.graphics.setShader()
	love.graphics.setCanvas()

	alt = not alt
	mx, my = love.mouse.getPosition()


	--GUI
	drawGUI()
	love.graphics.setColor(255, 255, 255, 255)
	--love.graphics.setShader(wave)
	love.graphics.draw(canvas[c2], 0, boxHeight, 0, scale, scale)
	--love.graphics.setShader()
	love.graphics.draw(canvas[3], 0)
	
	--Frame limiter
	local ct = love.timer.getTime()
	if nt <= ct then
		nt = ct
		return
	end
	love.timer.sleep(nt - ct)
end

function love.resize(w, h)
	screen.width = w
	screen.height = h
	createGUI()
	canvas[3] = love.graphics.newCanvas(screen.width, screen.height)
	applySettings(true)
end

function love.keypressed(key)
	if key == "escape" then love.event.push("quit") end
	if key == "space" then toggleSimulation() end
	if key == "s" then
		sim = true
		love.draw()
		sim = false
	end
	if key == "f1" then
		screenshot()
	end
	if key == "n" then
		noise()
	end
end

function love.mousepressed(x, y, key)

end

function screenshot()
	local s = love.graphics.newScreenshot()
	local n = os.date("%x")
	s:encode("png", os.time()..".png")
end

wave = love.graphics.newShader([[
	extern vec2 size;
	extern number t;
	vec4 effect(vec4 color, Image tex, vec2 tc, vec2 sc) 
	{
		tc.x = tc.x + sin((tc.y * 10.0) + t) * (size.x * 0.00001);
		tc.y = tc.y + sin((tc.x * 10.0) + t) * (size.y * 0.00001);
		vec4 pixel = Texel(tex, tc);

		return pixel;
	}
]])

shader = love.graphics.newShader([[
		extern Image map;
		extern vec2 size;
		extern vec4 aliveColor;
		extern vec4 deadColor;
		vec4 effect(vec4 color, Image tex, vec2 tc, vec2 sc) 
		{
			vec4 pixel = Texel(map, tc);
			vec4 aPixel = Texel(map, vec2(tc.x, tc.y - (1.0 / size.y)));
			vec4 bPixel = Texel(map, vec2(tc.x, tc.y + (1.0 / size.y)));
			vec4 lPixel = Texel(map, vec2(tc.x + (1.0 / size.x), tc.y));
			vec4 rPixel = Texel(map, vec2(tc.x - (1.0 / size.x), tc.y));
			vec4 tlPixel = Texel(map, vec2(tc.x - (1.0 / size.x), tc.y - (1.0 / size.y)));
			vec4 trPixel = Texel(map, vec2(tc.x + (1.0 / size.x), tc.y - (1.0 / size.y)));
			vec4 blPixel = Texel(map, vec2(tc.x - (1.0 / size.x), tc.y + (1.0 / size.y)));
			vec4 brPixel = Texel(map, vec2(tc.x + (1.0 / size.x), tc.y + (1.0 / size.y)));

			number n = 0.0;

			vec4 alive = aliveColor / vec4(255.0, 255.0, 255.0, 255.0);
			vec4 dead = deadColor / vec4(255.0, 255.0, 255.0, 255.0);

			if (aPixel == alive) {
				n++;
			}
			if (bPixel == alive) {
				n++;
			}
			if (lPixel == alive) {
				n++;
			}
			if (rPixel == alive) {
				n++;
			}
			if (tlPixel == alive) {
				n++;
			}
			if (trPixel == alive) {
				n++;
			}
			if (blPixel == alive) {
				n++;
			}
			if (brPixel == alive) {
				n++;
			}

			if (pixel == alive) {
				if (n == 2.0 || n == 3.0) {
					pixel = alive;
				} else {
					pixel = dead;
				};
			} else {
				if (n == 3.0) {
					pixel = alive;
				} else {
					pixel = dead;
				};
			};

			return pixel;
		}
	]])