local slider = {}
local sm = {__index = slider}

local ALL = {}

--LOCAL METHODS
local function normal(val, min, max)
	return (val - min) / (max - min)
end

local function lerp(norm, min, max)
	return (max - min) * norm + min
end

local function clamp(val, min, max)
	return math.max(min, math.min(val, max))
end
	
function slider.new(x, y, width, height, value, minValue, maxValue, floor)
	floor = floor or false
	local s = {
		x = x,
		y = y,
		width = width,
		height = height,
		value = value,
		minValue = minValue,
		maxValue = maxValue,
		floor = floor,
		selected = false
	}
	ALL[#ALL + 1] = s
	return setmetatable(s, sm)
end

function slider:update(dt)
	local mx, my = love.mouse.getPosition()
	local md = love.mouse.isDown(1)

	local other = false
	for i,v in ipairs(ALL) do 
		if v.selected then
			other = true
		end
	end

	if md then
		if mx > self.x and mx < self.x + self.width and my > self.y and my < self.y + self.height then
			if not other then self.selected = true end
		end
	else
		self.selected = false
	end

	if self.selected then
		local nx = normal(mx, self.x, self.x + self.width)
		local val = lerp(nx, self.minValue, self.maxValue)
		if self.floor then val = math.floor(val) end
		self.value = clamp(val, self.minValue, self.maxValue)
	end
end

function slider:draw()
	local ls = love.graphics.getLineStyle()
	love.graphics.setLineStyle("rough")
	love.graphics.rectangle("fill", self.x, self.y, self.width * self:getNormalValue(), self.height)
	love.graphics.rectangle("line", self.x, self.y, self.width, self.height)
	love.graphics.setLineStyle(ls)
end

--Getters
function slider:getValue()
	return self.value
end

function slider:getNormalValue()
	return normal(self.value, self.minValue, self.maxValue)
end

function slider:getPosition(axis)
	axis = axis or "both"
	if axis == "x" then
		return self.x
	elseif axis == "y" then
		return self.y 
	else
		return self.x, self.y
	end
end

function slider:getWidth()
	return self.width
end

function slider:getHeight()
	return self.height
end

--Setters
function slider:setValue(val)
	self.value = clamp(val, self.minValue, self.maxValue)
end

function slider:setNormalValue(val)
	self.value = clamp(lerp(val, self.minValue, self.maxValue), self.minValue, self.maxValue)
end

function slider:setPosition(x, y)
	x = x or self.x
	y = y or self.y
	self.x = x
	self.y = y
end

function slider:setWidth(width)
	width = width or self.width
	self.width = math.abs(width)
end

function slider:setHeight(height)
	height = height or self.height
	self.height = math.abs(height)
end

return slider